@@ -32,6 +32,7 @@ from manila.common import constants
 from manila import db
 from manila import exception
 from manila.i18n import _
+from manila.lock import api as resource_locks
 from manila import share
 from manila.share import share_types
 from manila import utils
@@ -51,6 +52,7 @@ class ShareMixin(object):
     def _delete(self, *args, **kwargs):
         return self.share_api.delete(*args, **kwargs)
 
+    @wsgi.Controller.authorize('get')
     def show(self, req, id):
         """Return data about the given share."""
         context = req.environ['manila.context']
@@ -62,6 +64,7 @@ class ShareMixin(object):
 
         return self._view_builder.detail(req, share)
 
+    @wsgi.Controller.authorize
     def delete(self, req, id):
         """Delete a share."""
         context = req.environ['manila.context']
@@ -90,12 +93,13 @@ class ShareMixin(object):
         except exception.NotFound:
             raise exc.HTTPNotFound()
         except exception.InvalidShare as e:
-            raise exc.HTTPForbidden(explanation=e.message)
+            raise exc.HTTPForbidden(explanation=e.msg)
         except exception.Conflict as e:
-            raise exc.HTTPConflict(explanation=e.message)
+            raise exc.HTTPConflict(explanation=e.msg)
 
         return webob.Response(status_int=http_client.ACCEPTED)
 
+    @wsgi.Controller.authorize("get_all")
     def index(self, req):
         """Returns a summary list of shares."""
         req.GET.pop('export_location_id', None)
@@ -106,6 +110,7 @@ class ShareMixin(object):
         req.GET.pop('with_count', None)
         return self._get_shares(req, is_detail=False)
 
+    @wsgi.Controller.authorize("get_all")
     def detail(self, req):
         """Returns a detailed list of shares."""
         req.GET.pop('export_location_id', None)
@@ -218,6 +223,11 @@ class ShareMixin(object):
                        for key in valid_update_keys
                        if key in share_data}
 
+        common.check_display_field_length(
+            update_dict.get('display_name'), 'display_name')
+        common.check_display_field_length(
+            update_dict.get('display_description'), 'display_description')
+
         try:
             share = self.share_api.get(context, id)
         except exception.NotFound:
@@ -258,12 +268,15 @@ class ShareMixin(object):
         # NOTE(rushiagr): Manila API allows 'name' instead of 'display_name'.
         if share.get('name'):
             share['display_name'] = share.get('name')
+            common.check_display_field_length(share['display_name'], 'name')
             del share['name']
 
         # NOTE(rushiagr): Manila API allows 'description' instead of
         #                 'display_description'.
         if share.get('description'):
             share['display_description'] = share.get('description')
+            common.check_display_field_length(
+                share['display_description'], 'description')
             del share['description']
 
         size = share['size']
@@ -282,14 +295,14 @@ class ShareMixin(object):
                 availability_zone_id = availability_zone_db.id
                 availability_zone = availability_zone_db.name
             except exception.AvailabilityZoneNotFound as e:
-                raise exc.HTTPNotFound(explanation=e.message)
+                raise exc.HTTPNotFound(explanation=e.msg)
 
         share_group_id = share.get('share_group_id')
         if share_group_id:
             try:
                 share_group = db.share_group_get(context, share_group_id)
             except exception.ShareGroupNotFound as e:
-                raise exc.HTTPNotFound(explanation=e.message)
+                raise exc.HTTPNotFound(explanation=e.msg)
             sg_az_id = share_group['availability_zone_id']
             if availability_zone and availability_zone_id != sg_az_id:
                 msg = _("Share cannot have AZ ('%(s_az)s') different than "
@@ -388,6 +401,8 @@ class ShareMixin(object):
                     exception.ShareTypeNotFoundByName):
                 msg = _("Share type not found.")
                 raise exc.HTTPNotFound(explanation=msg)
+            except exception.InvalidShareType as e:
+                raise exc.HTTPBadRequest(explanation=e.message)
         elif not snapshot:
             def_share_type = share_types.get_default_share_type()
             if def_share_type:
@@ -447,10 +462,61 @@ class ShareMixin(object):
                 return True
         return False
 
+    def _create_access_locks(
+            self, context, access, lock_deletion=False, lock_visibility=False,
+            lock_reason=None):
+        """Creates locks for access rules and rollback if it fails."""
+
+        # We must populate project_id and user_id in the access object, as this
+        # is not in this entity
+        access['project_id'] = context.project_id
+        access['user_id'] = context.user_id
+
+        def raise_lock_failed(access, lock_action):
+            word_mapping = {
+                constants.RESOURCE_ACTION_SHOW: 'visibility',
+                constants.RESOURCE_ACTION_DELETE: 'deletion'
+            }
+            msg = _("Failed to lock the %(action)s of the access rule "
+                    "%(rule)s.") % {
+                'action': word_mapping[lock_action],
+                'rule': access['id']
+            }
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+        deletion_lock = {}
+
+        if lock_deletion:
+            try:
+                deletion_lock = self.resource_locks_api.create(
+                    context, resource_id=access['id'],
+                    resource_type='access_rule',
+                    resource_action=constants.RESOURCE_ACTION_DELETE,
+                    resource=access, lock_reason=lock_reason)
+            except Exception:
+                raise_lock_failed(access, constants.RESOURCE_ACTION_DELETE)
+
+        if lock_visibility:
+            try:
+                self.resource_locks_api.create(
+                    context, resource_id=access['id'],
+                    resource_type='access_rule',
+                    resource_action=constants.RESOURCE_ACTION_SHOW,
+                    resource=access, lock_reason=lock_reason)
+            except Exception:
+                # If a deletion lock was placed and the visibility wasn't,
+                # we should rollback the deletion lock.
+                if deletion_lock:
+                    self.resource_locks_api.delete(
+                        context, deletion_lock['id'])
+                raise_lock_failed(access, constants.RESOURCE_ACTION_SHOW)
+
     @wsgi.Controller.authorize('allow_access')
     def _allow_access(self, req, id, body, enable_ceph=False,
                       allow_on_error_status=False, enable_ipv6=False,
-                      enable_metadata=False):
+                      enable_metadata=False, allow_on_error_state=False,
+                      lock_visibility=False, lock_deletion=False,
+                      lock_reason=None):
         """Add share access rule."""
         context = req.environ['manila.context']
         access_data = body.get('allow_access', body.get('os-allow_access'))
@@ -479,6 +545,11 @@ class ShareMixin(object):
             }
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
+        if not (lock_visibility or lock_deletion) and lock_reason:
+            msg = _("Lock reason can only be specified when locking the "
+                    "visibility or the deletion of an access rule.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         access_type = access_data['access_type']
         access_to = access_data['access_to']
         common.validate_access(access_type=access_type,
@@ -488,7 +559,8 @@ class ShareMixin(object):
         try:
             access = self.share_api.allow_access(
                 context, share, access_type, access_to,
-                access_data.get('access_level'), access_data.get('metadata'))
+                access_data.get('access_level'), access_data.get('metadata'),
+                allow_on_error_state)
         except exception.ShareAccessExists as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
 
@@ -498,15 +570,71 @@ class ShareMixin(object):
         except exception.InvalidMetadataSize as error:
             raise exc.HTTPBadRequest(explanation=error.msg)
 
+        if lock_deletion or lock_visibility:
+            self._create_access_locks(
+                context, access, lock_deletion=lock_deletion,
+                lock_visibility=lock_visibility, lock_reason=lock_reason)
+
         return self._access_view_builder.view(req, access)
 
+    def _check_for_access_rule_locks(self, context, access_data, access_id,
+                                     share_id):
+        """Fetches locks for access rules and attempts deleting them."""
+
+        # ensure the requester is asking to remove the restrictions of the rule
+        unrestrict = access_data.get('unrestrict', False)
+        search_opts = {
+            'resource_id': access_id,
+            'resource_action': constants.RESOURCE_ACTION_DELETE,
+            'all_projects': True,
+        }
+
+        locks, locks_count = (
+            self.resource_locks_api.get_all(
+                context.elevated(), search_opts=search_opts,
+                show_count=True) or []
+        )
+
+        # no locks placed, nothing to do
+        if not locks:
+            return
+
+        def raise_rule_is_locked(share_id, unrestrict=False):
+            msg = _(
+                "Cannot deny access for share '%s' since it has been "
+                "locked. Please remove the locks and retry the "
+                "operation") % share_id
+            if unrestrict:
+                msg = _(
+                    "Unable to drop access rule restrictions that are not "
+                    "placed by you.")
+            raise exc.HTTPForbidden(explanation=msg)
+
+        if locks_count and not unrestrict:
+            raise_rule_is_locked(share_id)
+
+        non_deletable_locks = []
+        for lock in locks:
+            try:
+                self.resource_locks_api.ensure_context_can_delete_lock(
+                    context, lock['id'])
+            except (exception.NotAuthorized, exception.ResourceLockNotFound):
+                # If it is not found, then it means that the context doesn't
+                # have access to this resource and should be denied.
+                non_deletable_locks.append(lock)
+
+        if non_deletable_locks:
+            raise_rule_is_locked(share_id, unrestrict=unrestrict)
+
     @wsgi.Controller.authorize('deny_access')
-    def _deny_access(self, req, id, body):
+    def _deny_access(self, req, id, body, allow_on_error_state=False):
         """Remove share access rule."""
         context = req.environ['manila.context']
 
-        access_id = body.get(
-            'deny_access', body.get('os-deny_access'))['access_id']
+        access_data = body.get('deny_access', body.get('os-deny_access'))
+        access_id = access_data['access_id']
+
+        self._check_for_access_rule_locks(context, access_data, access_id, id)
 
         share = self.share_api.get(context, id)
 
@@ -528,7 +656,8 @@ class ShareMixin(object):
             share = self.share_api.get(context, id)
         except exception.NotFound as error:
             raise webob.exc.HTTPNotFound(explanation=error.message)
-        self.share_api.deny_access(context, share, access)
+        self.share_api.deny_access(context, share, access,
+                                   allow_on_error_state)
         return webob.Response(status_int=http_client.ACCEPTED)
 
     def _access_list(self, req, id, body):
@@ -540,6 +669,7 @@ class ShareMixin(object):
 
         return self._access_view_builder.list_view(req, access_rules)
 
+    @wsgi.Controller.authorize("extend")
     def _extend(self, req, id, body):
         """Extend size of a share."""
         context = req.environ['manila.context']
@@ -560,6 +690,7 @@ class ShareMixin(object):
 
         return webob.Response(status_int=http_client.ACCEPTED)
 
+    @wsgi.Controller.authorize("shrink")
     def _shrink(self, req, id, body):
         """Shrink size of a share."""
         context = req.environ['manila.context']
@@ -627,6 +758,7 @@ class ShareController(wsgi.Controller, ShareMixin, wsgi.AdminActionsMixin):
     def __init__(self):
         super(ShareController, self).__init__()
         self.share_api = share.API()
+        self.resource_locks_api = resource_locks.API()
         self._access_view_builder = share_access_views.ViewBuilder()
 
     @wsgi.action('os-reset_status')
