@@ -25,6 +25,7 @@ from sqlalchemy import Column, Integer, String, schema
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
 from sqlalchemy import ForeignKey, DateTime, Boolean, Enum
+from sqlalchemy_utils import generic_repr
 
 from manila.common import constants
 
@@ -32,6 +33,7 @@ CONF = cfg.CONF
 BASE = declarative_base()
 
 
+@generic_repr
 class ManilaBase(models.ModelBase,
                  models.TimestampMixin,
                  models.SoftDeleteMixin):
@@ -63,6 +65,7 @@ class Service(BASE, ManilaBase):
     host = Column(String(255))  # , ForeignKey('hosts.id'))
     binary = Column(String(255))
     topic = Column(String(255))
+    state = Column(String(36))
     report_count = Column(Integer, nullable=False, default=0)
     disabled = Column(Boolean, default=False)
     availability_zone_id = Column(String(36),
@@ -302,6 +305,7 @@ class Share(BASE, ManilaBase):
     display_name = Column(String(255))
     display_description = Column(String(255))
     snapshot_id = Column(String(36))
+    source_backup_id = Column(String(36))
     snapshot_support = Column(Boolean, default=True)
     create_share_from_snapshot_support = Column(Boolean, default=True)
     revert_to_snapshot_support = Column(Boolean, default=False)
@@ -573,6 +577,15 @@ class ShareAccessMapping(BASE, ManilaBase):
             'ShareInstanceAccessMapping.deleted == "False")'
         )
     )
+    share = orm.relationship(
+        "Share",
+        primaryjoin=(
+            'and_('
+            'ShareAccessMapping.share_id == '
+            'Share.id, '
+            'Share.deleted == "False")'
+            )
+    )
 
 
 class ShareAccessRulesMetadata(BASE, ManilaBase):
@@ -931,6 +944,7 @@ class SecurityService(BASE, ManilaBase):
     name = Column(String(255), nullable=True)
     description = Column(String(255), nullable=True)
     ou = Column(String(255), nullable=True)
+    default_ad_site = Column(String(255), nullable=True)
 
 
 class ShareNetwork(BASE, ManilaBase):
@@ -1000,7 +1014,7 @@ class ShareNetwork(BASE, ManilaBase):
 class ShareNetworkSubnet(BASE, ManilaBase):
     """Represents a share network subnet used by some resources."""
 
-    _extra_keys = ['availability_zone']
+    _extra_keys = ['availability_zone', 'subnet_metadata']
 
     __tablename__ = 'share_network_subnets'
     id = Column(String(36), primary_key=True, nullable=False)
@@ -1055,6 +1069,39 @@ class ShareNetworkSubnet(BASE, ManilaBase):
     def share_network_name(self):
         return self.share_network['name']
 
+    @property
+    def subnet_metadata(self):
+        metadata_dict = {}
+        metadata_list = (
+            self.share_network_subnet_metadata)  # pylint: disable=no-member
+        for meta in metadata_list:
+            metadata_dict[meta['key']] = meta['value']
+        return metadata_dict
+
+    @property
+    def project_id(self):
+        return self.share_network['project_id']
+
+
+class ShareNetworkSubnetMetadata(BASE, ManilaBase):
+    """Represents a metadata key/value pair for a subnet."""
+    __tablename__ = 'share_network_subnet_metadata'
+    id = Column(Integer, primary_key=True)
+    key = Column(String(255), nullable=False)
+    value = Column(String(1023), nullable=False)
+    deleted = Column(String(36), default='False')
+    share_network_subnet_id = Column(String(36), ForeignKey(
+        'share_network_subnets.id'), nullable=False)
+
+    share_network_subnet = orm.relationship(
+        ShareNetworkSubnet,
+        backref=orm.backref('share_network_subnet_metadata', lazy='immediate'),
+        foreign_keys=share_network_subnet_id,
+        primaryjoin='and_('
+        'ShareNetworkSubnetMetadata.share_network_subnet_id == '
+        'ShareNetworkSubnet.id,'
+        'ShareNetworkSubnetMetadata.deleted == "False")')
+
 
 class ShareServer(BASE, ManilaBase):
     """Represents share server used by share."""
@@ -1169,6 +1216,26 @@ class ShareNetworkSecurityServiceAssociation(BASE, ManilaBase):
                                  nullable=False)
 
 
+class Transfer(BASE, ManilaBase):
+    """Represents a share transfer request."""
+
+    __tablename__ = 'transfers'
+
+    id = Column(String(36), primary_key=True, nullable=False)
+    deleted = Column(String(36), default='False')
+    # resource type can be "share" or "share_network"
+    resource_type = Column(String(36), nullable=False)
+    # The uuid of the related resource.
+    resource_id = Column(String(36), nullable=False)
+    display_name = Column(String(255))
+    salt = Column(String(255))
+    crypt_hash = Column(String(255))
+    expires_at = Column(DateTime)
+    source_project_id = Column(String(255), nullable=True)
+    destination_project_id = Column(String(255), nullable=True)
+    accepted = Column(Boolean, default=False)
+
+
 class NetworkAllocation(BASE, ManilaBase):
     """Represents network allocation data."""
     __tablename__ = 'network_allocations'
@@ -1202,6 +1269,9 @@ class DriverPrivateData(BASE, ManilaBase):
 class AvailabilityZone(BASE, ManilaBase):
     """Represents a private data as key-value pairs for a driver."""
     __tablename__ = 'availability_zones'
+    __table_args__ = (
+        schema.UniqueConstraint('name', 'deleted', name='az_name_uc'),
+    )
     id = Column(String(36), primary_key=True, nullable=False)
     deleted = Column(String(36), default='False')
     name = Column(String(255), nullable=False)
@@ -1394,6 +1464,31 @@ class Message(BASE, ManilaBase):
     deleted = Column(String(36), default='False')
 
 
+class ResourceLock(BASE, ManilaBase):
+    """Represents a resource lock.
+
+    Resource locks are held by users (or on behalf of users) and prevent
+    actions to be performed on resources while the lock is present.
+    """
+    __tablename__ = 'resource_locks'
+    id = Column(String(36), primary_key=True, nullable=False)
+    user_id = Column(String(255), nullable=False)
+    project_id = Column(String(255), nullable=False)
+    # If the lock is held on behalf of the user, but created by 'service' or
+    # 'admin' users, as opposed to the user themselves ('project')
+    lock_context = Column(String(10), nullable=False)
+    # The uuid of the resource being locked.
+    resource_id = Column(String(36), nullable=False)
+    # The resource type, a constant dict will hold possible values
+    resource_type = Column(Enum(*constants.RESOURCE_LOCK_RESOURCE_TYPES),
+                           default=constants.SHARE_RESOURCE_TYPE)
+    # Action that lock prevents, a constant dict will hold possible values
+    resource_action = Column(Enum(*constants.RESOURCE_LOCK_RESOURCE_ACTIONS),
+                             default=constants.RESOURCE_ACTION_DELETE)
+    lock_reason = Column(String(1023), nullable=True)
+    deleted = Column(String(36), default='False')
+
+
 class BackendInfo(BASE, ManilaBase):
     """Represent Backend Info."""
     __tablename__ = "backend_info"
@@ -1409,6 +1504,50 @@ class AsynchronousOperationData(BASE, ManilaBase):
     value = Column(String(1023), nullable=False)
 
 
+class ShareBackup(BASE, ManilaBase):
+    """Represents a backup of a share."""
+    __tablename__ = 'share_backups'
+    id = Column(String(36), primary_key=True)
+
+    @property
+    def name(self):
+        return CONF.share_backup_name_template % self.id
+
+    @property
+    def availability_zone(self):
+        if self._availability_zone:
+            return self._availability_zone['name']
+
+    deleted = Column(String(36), default='False')
+    user_id = Column(String(255), nullable=False)
+    project_id = Column(String(255), nullable=False)
+
+    share_id = Column(String(36), ForeignKey('shares.id'))
+    size = Column(Integer)
+    host = Column(String(255))
+    topic = Column(String(255))
+    display_name = Column(String(255))
+    display_description = Column(String(255))
+    progress = Column(String(32))
+    restore_progress = Column(String(32))
+    status = Column(String(255))
+    fail_reason = Column(String(1023))
+    availability_zone_id = Column(String(36),
+                                  ForeignKey('availability_zones.id'),
+                                  nullable=True)
+
+    _availability_zone = orm.relationship(
+        "AvailabilityZone",
+        lazy='immediate',
+        primaryjoin=(
+            'and_('
+            'ShareBackup.availability_zone_id == '
+            'AvailabilityZone.id, '
+            'AvailabilityZone.deleted == \'False\')'
+        )
+    )
+
+
 def register_models():
     """Register Models and create metadata.
 
