@@ -29,12 +29,15 @@ from manila import context
 from manila import db
 from manila import exception
 from manila.i18n import _
+from manila import quota
 
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
+QUOTAS = quota.QUOTAS
 
 MIN_SIZE_KEY = "provisioning:min_share_size"
 MAX_SIZE_KEY = "provisioning:max_share_size"
+MAX_EXTEND_SIZE_KEY = "provisioning:max_share_extend_size"
 
 
 def create(context, name, extra_specs=None, is_public=True,
@@ -356,7 +359,8 @@ def is_valid_optional_extra_spec(key, value):
     elif key == constants.ExtraSpecs.AVAILABILITY_ZONES:
         return is_valid_csv(value)
     elif key in [constants.ExtraSpecs.PROVISIONING_MAX_SHARE_SIZE,
-                 constants.ExtraSpecs.PROVISIONING_MIN_SHARE_SIZE]:
+                 constants.ExtraSpecs.PROVISIONING_MIN_SHARE_SIZE,
+                 constants.ExtraSpecs.PROVISIONING_MAX_SHARE_EXTEND_SIZE]:
         try:
             common.validate_integer(value, 'share_size', min_value=1)
             return True
@@ -433,7 +437,7 @@ def parse_boolean_extra_spec(extra_spec_key, extra_spec_value):
         raise exception.InvalidExtraSpec(reason=msg)
 
 
-def provision_filter_on_size(context, share_type, size):
+def provision_filter_on_size(context, share_type, size, operation='create'):
     """This function filters share provisioning requests on size limits.
 
     If a share type has provisioning size min/max set, this filter
@@ -442,9 +446,12 @@ def provision_filter_on_size(context, share_type, size):
     """
     if not share_type:
         share_type = get_default_share_type()
-    if share_type:
-        size_int = int(size)
-        extra_specs = share_type.get('extra_specs', {})
+        if not share_type:
+            return
+
+    size_int = int(size)
+    extra_specs = share_type.get('extra_specs', {})
+    if operation in ['create', 'shrink']:
         min_size = extra_specs.get(MIN_SIZE_KEY)
         if min_size and size_int < int(min_size):
             msg = _("Specified share size of '%(req_size)d' is less "
@@ -453,6 +460,7 @@ def provision_filter_on_size(context, share_type, size):
                     ) % {'req_size': size_int, 'min_size': min_size,
                          'sha_type': share_type['name']}
             raise exception.InvalidInput(reason=msg)
+    if operation in ['create', 'extend']:
         max_size = extra_specs.get(MAX_SIZE_KEY)
         if max_size and size_int > int(max_size):
             msg = _("Specified share size of '%(req_size)d' is "
@@ -461,3 +469,58 @@ def provision_filter_on_size(context, share_type, size):
                     ) % {'req_size': size_int, 'max_size': max_size,
                          'sha_type': share_type['name']}
             raise exception.InvalidInput(reason=msg)
+    if operation in ['admin-extend']:
+        max_extend_size = extra_specs.get(MAX_EXTEND_SIZE_KEY)
+        if max_extend_size and size_int > int(max_extend_size):
+            msg = _("Specified share size of '%(req_size)d' is "
+                    "greater than the maximum allowable extend size of "
+                    "'%(max_extend_size)s' for share type '%(sha_type)s'."
+                    ) % {'req_size': size_int,
+                         'max_extend_size': max_extend_size,
+                         'sha_type': share_type['name']}
+            raise exception.InvalidInput(reason=msg)
+
+
+def revert_allocated_share_type_quotas_during_migration(
+        context, share, share_type_id,
+        allow_deallocate_from_current_type=False):
+
+    # If both new share type and share's share type ID, there is no need
+    # to revert quotas because new quotas weren't allocated, as share
+    # type changes weren't identified, unless it is a migration that was
+    # successfully completed
+    if ((share_type_id == share['share_type_id'])
+            and not allow_deallocate_from_current_type):
+        return
+
+    new_share_type = get_share_type(context, share_type_id)
+    new_type_extra_specs = new_share_type.get('extra_specs', None)
+
+    new_type_replication_type = None
+    if new_type_extra_specs:
+        new_type_replication_type = new_type_extra_specs.get(
+            'replication_type', None)
+
+    deltas = {}
+
+    if new_type_replication_type:
+        deltas['share_replicas'] = -1
+        deltas['replica_gigabytes'] = -share['size']
+
+    deltas.update({
+        'share_type_id': new_share_type['id'],
+        'shares': -1,
+        'gigabytes': -share['size']
+    })
+
+    try:
+        reservations = QUOTAS.reserve(
+            context, project_id=share['project_id'],
+            user_id=share['user_id'], **deltas)
+    except Exception:
+        LOG.exception("Failed to update usages for share_replicas and "
+                      "replica_gigabytes.")
+    else:
+        QUOTAS.commit(
+            context, reservations, project_id=share['project_id'],
+            user_id=share['user_id'], share_type_id=share_type_id)
