@@ -29,8 +29,10 @@ from manila.common import constants
 from manila import context
 from manila import db
 from manila import exception
+from manila import quota
 from manila.share import share_types
 from manila import test
+from manila.tests import db_utils
 
 
 def create_share_type_dict(extra_specs=None):
@@ -461,7 +463,7 @@ class ShareTypesTestCase(test.TestCase):
             extra_specs[constants.ExtraSpecs.AVAILABILITY_ZONES] = spec_value
             expected_specs['availability_zones'] = 'az 1,az2,az 3'
 
-        self.assertDictMatch(expected_specs,
+        self.assertDictEqual(expected_specs,
                              share_types.sanitize_extra_specs(extra_specs))
 
     def test_add_access(self):
@@ -526,3 +528,125 @@ class ShareTypesTestCase(test.TestCase):
                           share_types.parse_boolean_extra_spec,
                           'fake_key',
                           spec_value)
+
+    def test_provision_filter_on_size(self):
+        share_types.create(self.context, "type1",
+                           extra_specs={
+                               "key1": "val1",
+                               "key2": "val2",
+                               "driver_handles_share_servers": False})
+        share_types.create(self.context, "type2",
+                           extra_specs={
+                               share_types.MIN_SIZE_KEY: "12",
+                               "key3": "val3",
+                               "driver_handles_share_servers": False})
+        share_types.create(self.context, "type3",
+                           extra_specs={
+                               share_types.MAX_SIZE_KEY: "99",
+                               "key4": "val4",
+                               "driver_handles_share_servers": False})
+        share_types.create(self.context, "type4",
+                           extra_specs={
+                               share_types.MIN_SIZE_KEY: "24",
+                               share_types.MAX_SIZE_KEY: "99",
+                               "key4": "val4",
+                               "driver_handles_share_servers": False})
+        share_types.create(self.context, "type5",
+                           extra_specs={
+                               share_types.MAX_SIZE_KEY: "95",
+                               share_types.MAX_EXTEND_SIZE_KEY: "99",
+                               "key4": "val4",
+                               "driver_handles_share_servers": False})
+
+        # Make sure we don't raise if there are no min/max set
+        type1 = share_types.get_share_type_by_name(self.context, 'type1')
+        share_types.provision_filter_on_size(self.context, type1, "11")
+
+        # verify minimum size requirements
+        type2 = share_types.get_share_type_by_name(self.context, 'type2')
+        self.assertRaises(exception.InvalidInput,
+                          share_types.provision_filter_on_size,
+                          self.context, type2, "11")
+        share_types.provision_filter_on_size(self.context, type2, "12")
+        share_types.provision_filter_on_size(self.context, type2, "100")
+
+        # verify max size requirements
+        type3 = share_types.get_share_type_by_name(self.context, 'type3')
+        self.assertRaises(exception.InvalidInput,
+                          share_types.provision_filter_on_size,
+                          self.context, type3, "100")
+        share_types.provision_filter_on_size(self.context, type3, "99")
+        share_types.provision_filter_on_size(self.context, type3, "1")
+
+        # verify min and max
+        type4 = share_types.get_share_type_by_name(self.context, 'type4')
+        self.assertRaises(exception.InvalidInput,
+                          share_types.provision_filter_on_size,
+                          self.context, type4, "20")
+        self.assertRaises(exception.InvalidInput,
+                          share_types.provision_filter_on_size,
+                          self.context, type4, "100")
+        share_types.provision_filter_on_size(self.context, type4, "24")
+        share_types.provision_filter_on_size(self.context, type4, "99")
+        share_types.provision_filter_on_size(self.context, type4, "30")
+
+        # verify max extend size requirements
+        type5 = share_types.get_share_type_by_name(self.context, 'type5')
+        self.assertRaises(exception.InvalidInput,
+                          share_types.provision_filter_on_size,
+                          self.context, type5, "100", operation="extend")
+        share_types.provision_filter_on_size(self.context, type5, "99",
+                                             operation="admin-extend")
+
+    @ddt.data(True, False)
+    def test__revert_allocated_share_type_quotas_during_migration(
+            self, failed_on_reservation):
+        fake_type_id = 'fake_1'
+        extra_specs = {'replication_type': 'readable'}
+        source_instance = db_utils.create_share()
+        dest_instance = db_utils.create_share(
+            share_type_id=fake_type_id)
+        share_type = {
+            'name': 'fake_share_type',
+            'extra_specs': extra_specs,
+            'is_public': True,
+            'id': 'fake_type_id'
+        }
+
+        expected_deltas = {
+            'project_id': dest_instance['project_id'],
+            'user_id': dest_instance['user_id'],
+            'share_replicas': -1,
+            'replica_gigabytes': -dest_instance['size'],
+            'share_type_id': share_type['id'],
+            'shares': -1,
+            'gigabytes': -dest_instance['size'],
+        }
+        reservations = 'reservations'
+        reservation_action = (
+            mock.Mock(side_effect=exception.ManilaException(message='fake'))
+            if failed_on_reservation else mock.Mock(return_value=reservations))
+
+        mock_type_get = self.mock_object(
+            share_types, 'get_share_type', mock.Mock(return_value=share_type))
+        mock_reserve = self.mock_object(
+            quota.QUOTAS, 'reserve', reservation_action)
+        mock_commit = self.mock_object(quota.QUOTAS, 'commit')
+        mock_log = self.mock_object(share_types.LOG, 'exception')
+
+        share_types.revert_allocated_share_type_quotas_during_migration(
+            self.context, source_instance, share_type['id'], dest_instance)
+
+        if not failed_on_reservation:
+            mock_commit.assert_called_once_with(
+                self.context, reservations,
+                project_id=dest_instance['project_id'],
+                user_id=dest_instance['user_id'],
+                share_type_id=share_type['id'])
+        else:
+            mock_log.assert_called_once()
+
+        mock_type_get.assert_called_once_with(
+            self.context, share_type['id'])
+        mock_reserve.assert_called_once_with(
+            self.context, **expected_deltas)
