@@ -21,10 +21,9 @@ from unittest import mock
 
 import ddt
 from oslo_config import cfg
+from oslo_utils import encodeutils
 from oslo_utils import timeutils
-from oslo_utils import uuidutils
-import paramiko
-import six
+import tenacity
 from webob import exc
 
 import manila
@@ -195,171 +194,6 @@ class MonkeyPatchTestCase(test.TestCase):
                          manila.tests.monkey_patch_example.CALLED_FUNCTION)
 
 
-class FakeSSHClient(object):
-
-    def __init__(self):
-        self.id = uuidutils.generate_uuid()
-        self.transport = FakeTransport()
-
-    def set_missing_host_key_policy(self, policy):
-        pass
-
-    def connect(self, ip, port=22, username=None, password=None,
-                key_filename=None, look_for_keys=None, timeout=10,
-                banner_timeout=10):
-        pass
-
-    def get_transport(self):
-        return self.transport
-
-    def close(self):
-        pass
-
-    def __call__(self, *args, **kwargs):
-        pass
-
-
-class FakeSock(object):
-    def settimeout(self, timeout):
-        pass
-
-
-class FakeTransport(object):
-
-    def __init__(self):
-        self.active = True
-        self.sock = FakeSock()
-
-    def set_keepalive(self, timeout):
-        pass
-
-    def is_active(self):
-        return self.active
-
-
-class SSHPoolTestCase(test.TestCase):
-    """Unit test for SSH Connection Pool."""
-
-    def test_single_ssh_connect(self):
-        with mock.patch.object(paramiko, "SSHClient",
-                               mock.Mock(return_value=FakeSSHClient())):
-            sshpool = utils.SSHPool("127.0.0.1", 22, 10, "test",
-                                    password="test", min_size=1, max_size=1)
-            with sshpool.item() as ssh:
-                first_id = ssh.id
-
-            with sshpool.item() as ssh:
-                second_id = ssh.id
-
-            self.assertEqual(first_id, second_id)
-            paramiko.SSHClient.assert_called_once_with()
-
-    def test_create_ssh_with_password(self):
-        fake_ssh_client = mock.Mock()
-        ssh_pool = utils.SSHPool("127.0.0.1", 22, 10, "test",
-                                 password="test")
-        with mock.patch.object(paramiko, "SSHClient",
-                               return_value=fake_ssh_client):
-            ssh_pool.create()
-
-            fake_ssh_client.connect.assert_called_once_with(
-                "127.0.0.1", port=22, username="test",
-                password="test", key_filename=None, look_for_keys=False,
-                timeout=10, banner_timeout=10)
-
-    def test_create_ssh_with_key(self):
-        path_to_private_key = "/fakepath/to/privatekey"
-        fake_ssh_client = mock.Mock()
-        ssh_pool = utils.SSHPool("127.0.0.1", 22, 10, "test",
-                                 privatekey="/fakepath/to/privatekey")
-        with mock.patch.object(paramiko, "SSHClient",
-                               return_value=fake_ssh_client):
-            ssh_pool.create()
-            fake_ssh_client.connect.assert_called_once_with(
-                "127.0.0.1", port=22, username="test", password=None,
-                key_filename=path_to_private_key, look_for_keys=False,
-                timeout=10, banner_timeout=10)
-
-    def test_create_ssh_with_nothing(self):
-        fake_ssh_client = mock.Mock()
-        ssh_pool = utils.SSHPool("127.0.0.1", 22, 10, "test")
-        with mock.patch.object(paramiko, "SSHClient",
-                               return_value=fake_ssh_client):
-            ssh_pool.create()
-            fake_ssh_client.connect.assert_called_once_with(
-                "127.0.0.1", port=22, username="test", password=None,
-                key_filename=None, look_for_keys=True,
-                timeout=10, banner_timeout=10)
-
-    def test_create_ssh_error_connecting(self):
-        attrs = {'connect.side_effect': paramiko.SSHException, }
-        fake_ssh_client = mock.Mock(**attrs)
-        ssh_pool = utils.SSHPool("127.0.0.1", 22, 10, "test")
-        with mock.patch.object(paramiko, "SSHClient",
-                               return_value=fake_ssh_client):
-            self.assertRaises(exception.SSHException, ssh_pool.create)
-            fake_ssh_client.connect.assert_called_once_with(
-                "127.0.0.1", port=22, username="test", password=None,
-                key_filename=None, look_for_keys=True,
-                timeout=10, banner_timeout=10)
-
-    def test_closed_reopend_ssh_connections(self):
-        with mock.patch.object(paramiko, "SSHClient",
-                               mock.Mock(return_value=FakeSSHClient())):
-            sshpool = utils.SSHPool("127.0.0.1", 22, 10, "test",
-                                    password="test", min_size=1, max_size=2)
-            with sshpool.item() as ssh:
-                first_id = ssh.id
-            with sshpool.item() as ssh:
-                second_id = ssh.id
-                # Close the connection and test for a new connection
-                ssh.get_transport().active = False
-            self.assertEqual(first_id, second_id)
-            paramiko.SSHClient.assert_called_once_with()
-
-        # Expected new ssh pool
-        with mock.patch.object(paramiko, "SSHClient",
-                               mock.Mock(return_value=FakeSSHClient())):
-            with sshpool.item() as ssh:
-                third_id = ssh.id
-            self.assertNotEqual(first_id, third_id)
-            paramiko.SSHClient.assert_called_once_with()
-
-    @mock.patch('six.moves.builtins.open')
-    @mock.patch('paramiko.SSHClient')
-    @mock.patch('os.path.isfile', return_value=True)
-    def test_sshpool_remove(self, mock_isfile, mock_sshclient, mock_open):
-        ssh_to_remove = mock.Mock()
-        mock_sshclient.side_effect = [mock.Mock(), ssh_to_remove, mock.Mock()]
-        sshpool = utils.SSHPool("127.0.0.1", 22, 10, "test", password="test",
-                                min_size=3, max_size=3)
-
-        self.assertIn(ssh_to_remove, list(sshpool.free_items))
-
-        sshpool.remove(ssh_to_remove)
-
-        self.assertNotIn(ssh_to_remove, list(sshpool.free_items))
-
-    @mock.patch('six.moves.builtins.open')
-    @mock.patch('paramiko.SSHClient')
-    @mock.patch('os.path.isfile', return_value=True)
-    def test_sshpool_remove_object_not_in_pool(self, mock_isfile,
-                                               mock_sshclient, mock_open):
-        # create an SSH Client that is not a part of sshpool.
-        ssh_to_remove = mock.Mock()
-        mock_sshclient.side_effect = [mock.Mock(), mock.Mock()]
-
-        sshpool = utils.SSHPool("127.0.0.1", 22, 10, "test", password="test",
-                                min_size=2, max_size=2)
-        listBefore = list(sshpool.free_items)
-
-        self.assertNotIn(ssh_to_remove, listBefore)
-
-        sshpool.remove(ssh_to_remove)
-
-        self.assertEqual(listBefore, list(sshpool.free_items))
-
-
 @ddt.ddt
 class CidrToNetmaskTestCase(test.TestCase):
     """Unit test for cidr to netmask."""
@@ -557,12 +391,16 @@ class TestComparableMixin(test.TestCase):
                          self.one._compare(1, self.one._cmpkey))
 
 
+class WrongException(Exception):
+    pass
+
+
 class TestRetryDecorator(test.TestCase):
     def test_no_retry_required(self):
         self.counter = 0
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException,
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException,
                          interval=2,
                          retries=3,
                          backoff_rate=2)
@@ -578,8 +416,8 @@ class TestRetryDecorator(test.TestCase):
     def test_no_retry_required_random(self):
         self.counter = 0
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException,
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException,
                          interval=2,
                          retries=3,
                          backoff_rate=2,
@@ -593,42 +431,17 @@ class TestRetryDecorator(test.TestCase):
             self.assertEqual('success', ret)
             self.assertEqual(1, self.counter)
 
-    def test_retries_once_random(self):
-        self.counter = 0
-        interval = 2
-        backoff_rate = 2
-        retries = 3
-
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException,
-                         interval,
-                         retries,
-                         backoff_rate,
-                         wait_random=True)
-            def fails_once():
-                self.counter += 1
-                if self.counter < 2:
-                    raise exception.ManilaException(message='fake')
-                else:
-                    return 'success'
-
-            ret = fails_once()
-            self.assertEqual('success', ret)
-            self.assertEqual(2, self.counter)
-            self.assertEqual(1, mock_sleep.call_count)
-            self.assertTrue(mock_sleep.called)
-
     def test_retries_once(self):
         self.counter = 0
         interval = 2
         backoff_rate = 2
         retries = 3
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException,
-                         interval,
-                         retries,
-                         backoff_rate)
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException,
+                         interval=interval,
+                         retries=retries,
+                         backoff_rate=backoff_rate)
             def fails_once():
                 self.counter += 1
                 if self.counter < 2:
@@ -640,7 +453,32 @@ class TestRetryDecorator(test.TestCase):
             self.assertEqual('success', ret)
             self.assertEqual(2, self.counter)
             self.assertEqual(1, mock_sleep.call_count)
-            mock_sleep.assert_called_with(interval * backoff_rate)
+            mock_sleep.assert_called_with(interval)
+
+    def test_retries_once_random(self):
+        self.counter = 0
+        interval = 2
+        backoff_rate = 2
+        retries = 3
+
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException,
+                         interval=interval,
+                         retries=retries,
+                         backoff_rate=backoff_rate,
+                         wait_random=True)
+            def fails_once():
+                self.counter += 1
+                if self.counter < 2:
+                    raise exception.ManilaException(data='fake')
+                else:
+                    return 'success'
+
+            ret = fails_once()
+            self.assertEqual('success', ret)
+            self.assertEqual(2, self.counter)
+            self.assertEqual(1, mock_sleep.call_count)
+            self.assertTrue(mock_sleep.called)
 
     def test_limit_is_reached(self):
         self.counter = 0
@@ -648,11 +486,11 @@ class TestRetryDecorator(test.TestCase):
         interval = 2
         backoff_rate = 4
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException,
-                         interval,
-                         retries,
-                         backoff_rate)
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException,
+                         interval=interval,
+                         retries=retries,
+                         backoff_rate=backoff_rate)
             def always_fails():
                 self.counter += 1
                 raise exception.ManilaException(data='fake')
@@ -662,34 +500,94 @@ class TestRetryDecorator(test.TestCase):
             self.assertEqual(retries, self.counter)
 
             expected_sleep_arg = []
-
             for i in range(retries):
                 if i > 0:
-                    interval *= backoff_rate
+                    interval *= (backoff_rate ** (i - 1))
                     expected_sleep_arg.append(float(interval))
 
-            mock_sleep.assert_has_calls(map(mock.call, expected_sleep_arg))
+            mock_sleep.assert_has_calls(
+                list(map(mock.call, expected_sleep_arg)))
 
     def test_wrong_exception_no_retry(self):
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException)
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException)
             def raise_unexpected_error():
-                raise ValueError("value error")
+                raise WrongException("wrong exception")
 
-            self.assertRaises(ValueError, raise_unexpected_error)
+            self.assertRaises(WrongException, raise_unexpected_error)
             self.assertFalse(mock_sleep.called)
 
-    def test_wrong_retries_num(self):
-        self.assertRaises(ValueError, utils.retry, exception.ManilaException,
-                          retries=-1)
+    @mock.patch('tenacity.nap.sleep')
+    def test_retry_exit_code(self, sleep_mock):
+
+        exit_code = 5
+        exception = utils.processutils.ProcessExecutionError
+
+        @utils.retry(retry=utils.retry_if_exit_code, retry_param=exit_code)
+        def raise_retriable_exit_code():
+            raise exception(exit_code=exit_code)
+
+        self.assertRaises(exception, raise_retriable_exit_code)
+        # we should be sleeping 1 less time than the number of retries,
+        # default (10)
+        self.assertEqual(9, sleep_mock.call_count)
+        sleep_mock.assert_has_calls([mock.call(1.0),
+                                     mock.call(2.0),
+                                     mock.call(4.0),
+                                     mock.call(8.0),
+                                     mock.call(16.0),
+                                     mock.call(32.0),
+                                     mock.call(64.0),
+                                     mock.call(128.0),
+                                     mock.call(256.0)])
+
+    @mock.patch('tenacity.nap.sleep')
+    def test_retry_exit_code_non_retriable(self, sleep_mock):
+
+        exit_code = 5
+        exception = utils.processutils.ProcessExecutionError
+
+        @utils.retry(retry=utils.retry_if_exit_code, retry_param=exit_code)
+        def raise_non_retriable_exit_code():
+            raise exception(exit_code=exit_code + 1)
+
+        self.assertRaises(exception, raise_non_retriable_exit_code)
+        sleep_mock.assert_not_called()
+
+    def test_infinite_retry(self):
+        retry_param = exception.ManilaException
+
+        class FakeTenacityRetry(tenacity.Retrying):
+            def __init__(*args, **kwargs):
+                pass
+
+        with mock.patch('tenacity.Retrying',
+                        autospec=FakeTenacityRetry) as tenacity_retry:
+
+            @utils.retry(retry_param=retry_param,
+                         wait_random=True,
+                         infinite=True)
+            def some_retriable_function():
+                pass
+
+            some_retriable_function()
+
+            tenacity_retry.assert_called_once_with(
+                sleep=tenacity.nap.sleep,
+                before_sleep=mock.ANY,
+                after=mock.ANY,
+                stop=tenacity.stop.stop_never,
+                reraise=True,
+                retry=utils.IsAMatcher(tenacity.retry_if_exception_type),
+                wait=utils.IsAMatcher(tenacity.wait_random_exponential))
 
     def test_max_backoff_sleep(self):
         self.counter = 0
 
-        with mock.patch.object(time, 'sleep') as mock_sleep:
-            @utils.retry(exception.ManilaException,
-                         retries=0,
+        with mock.patch('tenacity.nap.sleep') as mock_sleep:
+            @utils.retry(retry_param=exception.ManilaException,
+                         infinite=True,
                          backoff_rate=2,
                          backoff_sleep_max=4)
             def fails_then_passes():
@@ -700,7 +598,8 @@ class TestRetryDecorator(test.TestCase):
                     return 'success'
 
             self.assertEqual('success', fails_then_passes())
-            mock_sleep.assert_has_calls(map(mock.call, [2, 4, 4, 4]))
+            mock_sleep.assert_has_calls(
+                [mock.call(1), mock.call(2), mock.call(4), mock.call(4)])
 
 
 @ddt.ddt
@@ -804,36 +703,23 @@ class ShareMigrationHelperTestCase(test.TestCase):
 class ConvertStrTestCase(test.TestCase):
 
     def test_convert_str_str_input(self):
-        self.mock_object(utils.encodeutils, 'safe_encode')
-        input_value = six.text_type("string_input")
+        self.mock_object(encodeutils, 'safe_encode')
+        input_value = "string_input"
 
         output_value = utils.convert_str(input_value)
 
-        if six.PY2:
-            utils.encodeutils.safe_encode.assert_called_once_with(input_value)
-            self.assertEqual(
-                utils.encodeutils.safe_encode.return_value, output_value)
-        else:
-            self.assertEqual(0, utils.encodeutils.safe_encode.call_count)
-            self.assertEqual(input_value, output_value)
+        self.assertEqual(0, encodeutils.safe_encode.call_count)
+        self.assertEqual(input_value, output_value)
 
     def test_convert_str_bytes_input(self):
-        self.mock_object(utils.encodeutils, 'safe_encode')
-        if six.PY2:
-            input_value = six.binary_type("binary_input")
-        else:
-            input_value = six.binary_type("binary_input", "utf-8")
+        self.mock_object(encodeutils, 'safe_encode')
+        input_value = bytes("binary_input", "utf-8")
 
         output_value = utils.convert_str(input_value)
 
-        if six.PY2:
-            utils.encodeutils.safe_encode.assert_called_once_with(input_value)
-            self.assertEqual(
-                utils.encodeutils.safe_encode.return_value, output_value)
-        else:
-            self.assertEqual(0, utils.encodeutils.safe_encode.call_count)
-            self.assertIsInstance(output_value, six.string_types)
-            self.assertEqual(six.text_type("binary_input"), output_value)
+        self.assertEqual(0, encodeutils.safe_encode.call_count)
+        self.assertIsInstance(output_value, str)
+        self.assertEqual(str("binary_input"), output_value)
 
 
 @ddt.ddt
